"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const Mito_1 = require("./Mito");
const DefaultModal_1 = __importDefault(require("./DefaultModal"));
// import css
require("../../css/merge-modal.css");
const getSharedKeys = (sheetOneIndex, sheetTwoIndex, sheetJSONArray) => {
    // Given two sheets indexes, returns all the columns that are shared between both sheets
    const sheetOneColumnNames = sheetJSONArray[sheetOneIndex].columns;
    const sheetTwoColumnNames = sheetJSONArray[sheetTwoIndex].columns;
    const sharedKeys = sheetOneColumnNames.filter(columnName => sheetTwoColumnNames.includes(columnName)).map(columnHeader => columnHeader.toString());
    return sharedKeys;
};
const getSuggestedKeys = (sheetOneIndex, sheetTwoIndex, sheetJSONArray) => {
    // Given two sheet indexes, reccomends a key to merge on. If there are x shared keys, then:
    // - If x.length == 1, it returns this key as reccomended.
    // - Else, it does not reccomend a key!
    const sharedKeys = getSharedKeys(sheetOneIndex, sheetTwoIndex, sheetJSONArray);
    const sheetOneMergeKey = sharedKeys.length === 1 ? sharedKeys[0] : sheetJSONArray[sheetOneIndex].columns[0];
    const sheetTwoMergeKey = sharedKeys.length === 1 ? sharedKeys[0] : sheetJSONArray[sheetTwoIndex].columns[0];
    return {
        sheetOneMergeKey: sheetOneMergeKey,
        sheetTwoMergeKey: sheetTwoMergeKey
    };
};
class MergeModal extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.updateSheetOneSelection = (e) => {
            const newSheetIndex = parseInt(e.target.value);
            const suggestedKeys = getSuggestedKeys(newSheetIndex, this.state.sheetTwoIndex, this.props.sheetJSONArray);
            this.setState({
                sheetOneIndex: newSheetIndex,
                sheetOneMergeKey: suggestedKeys.sheetOneMergeKey,
                sheetTwoMergeKey: suggestedKeys.sheetTwoMergeKey,
            });
        };
        this.updateSheetOneKeySelection = (e) => {
            const newSheetMergeKey = e.target.value;
            this.setState({ sheetOneMergeKey: newSheetMergeKey });
        };
        this.updateSheetTwoSelection = (e) => {
            const newSheetIndex = parseInt(e.target.value);
            const suggestedKeys = getSuggestedKeys(this.state.sheetOneIndex, newSheetIndex, this.props.sheetJSONArray);
            this.setState({
                sheetTwoIndex: newSheetIndex,
                sheetOneMergeKey: suggestedKeys.sheetOneMergeKey,
                sheetTwoMergeKey: suggestedKeys.sheetTwoMergeKey
            });
        };
        this.updateSheetTwoKeySelection = (e) => {
            const newSheetMergeKey = e.target.value;
            this.setState({ sheetTwoMergeKey: newSheetMergeKey });
        };
        this.completeMerge = () => {
            var _a;
            (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
                userId: window.user_id,
                event: 'button_merge_log_event',
                properties: {
                    stage: 'merged',
                    sheet_index_one: this.state.sheetOneIndex,
                    merge_key_one: this.state.sheetOneMergeKey,
                    sheet_index_two: this.state.sheetTwoIndex,
                    merge_key_two: this.state.sheetTwoMergeKey
                }
            });
            this.props.send({
                'event': 'edit_event',
                'type': 'merge',
                'id': '123',
                'timestamp': '456',
                'sheet_index_one': this.state.sheetOneIndex,
                'merge_key_one': this.state.sheetOneMergeKey,
                'sheet_index_two': this.state.sheetTwoIndex,
                'merge_key_two': this.state.sheetTwoMergeKey
            });
            this.props.setModal(Mito_1.ModalEnum.None);
        };
        const sheetOneIndex = 0;
        const sheetTwoIndex = Math.min(1, props.dfNames.length - 1);
        const suggestedKeys = getSuggestedKeys(sheetOneIndex, sheetTwoIndex, props.sheetJSONArray);
        this.state = {
            sheetOneIndex: sheetOneIndex,
            sheetTwoIndex: sheetTwoIndex,
            sheetOneMergeKey: suggestedKeys.sheetOneMergeKey,
            sheetTwoMergeKey: suggestedKeys.sheetTwoMergeKey,
        };
        this.updateSheetOneSelection = this.updateSheetOneSelection.bind(this);
        this.updateSheetOneKeySelection = this.updateSheetOneKeySelection.bind(this);
        this.updateSheetTwoSelection = this.updateSheetTwoSelection.bind(this);
        this.updateSheetTwoKeySelection = this.updateSheetTwoKeySelection.bind(this);
        this.completeMerge = this.completeMerge.bind(this);
    }
    render() {
        const sheetOneSelect = (react_1.default.createElement("select", { className: 'merge-modal-sheet-dropdown', onChange: this.updateSheetOneSelection }, this.props.dfNames.map((dfName, index) => {
            return (react_1.default.createElement("option", { key: dfName, value: index, selected: this.state.sheetOneIndex === index }, dfName));
        })));
        const sheetOneMergeKeySelect = (react_1.default.createElement("select", { className: 'merge-modal-key-dropdown', onChange: this.updateSheetOneKeySelection }, this.props.sheetJSONArray[this.state.sheetOneIndex].columns.map((columnHeader) => {
            return (react_1.default.createElement("option", { key: columnHeader, value: columnHeader, selected: this.state.sheetOneMergeKey === columnHeader }, columnHeader));
        })));
        const sheetTwoSelect = (react_1.default.createElement("select", { className: 'merge-modal-sheet-dropdown', onChange: this.updateSheetTwoSelection }, this.props.dfNames.map((dfName, index) => {
            return (react_1.default.createElement("option", { key: dfName, value: index, selected: this.state.sheetTwoIndex === index }, dfName));
        })));
        const sheetTwoMergeKeySelect = (react_1.default.createElement("select", { className: 'merge-modal-key-dropdown', onChange: this.updateSheetTwoKeySelection }, this.props.sheetJSONArray[this.state.sheetTwoIndex].columns.map((columnHeader) => {
            return (react_1.default.createElement("option", { key: columnHeader, value: columnHeader, selected: this.state.sheetTwoMergeKey === columnHeader }, columnHeader));
        })));
        // if there are no sheets to merge, don't try to display modal
        if (this.props.dfNames.length === 0) {
            return (react_1.default.createElement(DefaultModal_1.default, { header: 'Merge two sheets', modalType: Mito_1.ModalEnum.Merge, viewComponent: react_1.default.createElement(react_1.Fragment, null, "There are no sheets to merge"), buttons: react_1.default.createElement(react_1.Fragment, null,
                    react_1.default.createElement("div", { className: 'modal-close-button', onClick: () => { this.props.setModal(Mito_1.ModalEnum.None); } }, " Close ")) }));
        }
        return (react_1.default.createElement(DefaultModal_1.default, { header: 'Merge two sheets', modalType: Mito_1.ModalEnum.Merge, viewComponent: react_1.default.createElement(react_1.Fragment, null,
                react_1.default.createElement("div", { className: 'merge-modal-content' },
                    react_1.default.createElement("div", { className: 'merge-modal-sheet-and-key' },
                        react_1.default.createElement("div", null,
                            react_1.default.createElement("p", { className: 'merge-modal-sheet-label' }, "First Sheet"),
                            sheetOneSelect),
                        react_1.default.createElement("div", null,
                            react_1.default.createElement("p", { className: 'merge-modal-key-label' }, "Merge Key"),
                            sheetOneMergeKeySelect)),
                    react_1.default.createElement("div", { className: 'merge-modal-sheet-and-key' },
                        react_1.default.createElement("div", null,
                            react_1.default.createElement("p", { className: 'merge-modal-sheet-label' }, "Second Sheet"),
                            sheetTwoSelect),
                        react_1.default.createElement("div", null,
                            react_1.default.createElement("p", { className: 'merge-modal-key-label' }, "Merge Key"),
                            sheetTwoMergeKeySelect)))), buttons: react_1.default.createElement(react_1.Fragment, null,
                react_1.default.createElement("div", { className: 'modal-close-button modal-dual-button-left', onClick: () => { this.props.setModal(Mito_1.ModalEnum.None); } }, " Close "),
                react_1.default.createElement("div", { className: 'modal-action-button modal-dual-button-right', onClick: this.completeMerge },
                    " ",
                    "Merge")) }));
    }
}
exports.default = MergeModal;
//# sourceMappingURL=MergeModal.js.map