"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const DefaultTaskpane_1 = __importDefault(require("../../DefaultTaskpane"));
const LargeSelect_1 = __importDefault(require("../../elements/LargeSelect"));
const PivotTableKeySelection_1 = __importDefault(require("./PivotTableKeySelection"));
// import 
require("../../../../css/pivot-taskpane.css");
const PivotTableValueSelection_1 = __importDefault(require("./PivotTableValueSelection"));
// The aggregation functions we're supporting, out of the box
var AggregationType;
(function (AggregationType) {
    AggregationType["SUM"] = "sum";
    AggregationType["MEAN"] = "mean";
    AggregationType["MIN"] = "min";
    AggregationType["MAX"] = "max";
    AggregationType["COUNT"] = "count";
    AggregationType["NONE"] = "none";
})(AggregationType = exports.AggregationType || (exports.AggregationType = {}));
var KeyType;
(function (KeyType) {
    KeyType["Row"] = "Row";
    KeyType["Col"] = "Col";
})(KeyType = exports.KeyType || (exports.KeyType = {}));
var KeySelectionOperation;
(function (KeySelectionOperation) {
    KeySelectionOperation["ADD"] = "Add";
    KeySelectionOperation["REMOVE"] = "Remove";
})(KeySelectionOperation = exports.KeySelectionOperation || (exports.KeySelectionOperation = {}));
class PivotTaskpane extends react_1.default.Component {
    constructor(props) {
        super(props);
        /* create aggregated values mapping for columns with no aggregation */
        this.initializeAggregatedValues = (columnHeaders) => {
            const aggregatedValues = {};
            columnHeaders.forEach(columnHeader => {
                aggregatedValues[columnHeader] = AggregationType.NONE;
            });
            return aggregatedValues;
        };
        this.editValueAggregationSelection = (columnHeader, aggregationType) => {
            this.setState(prevState => {
                const newValues = prevState.values;
                newValues[columnHeader] = aggregationType;
                return {
                    values: newValues
                };
            });
        };
        /*
            a callback sent select sheet Select Element so that it can
            set the state of the Pivot Table Taskpane
    
        */
        this.setSelectedSheet = (newSheetName) => {
            const newSelectedSheetIndex = this.props.dfNames.indexOf(newSheetName);
            /*
                reset the keys so
                    1) the column headers currently selected are dropped
                    2) the values currently aggregated are dropped
                    3) display the column headers from the new sheet
            */
            const resetKeys = this.props.sheetJSONArray[newSelectedSheetIndex].columns.map(() => { return false; });
            this.setState({
                selectedSheetIndex: newSelectedSheetIndex,
                selectedGroupByRows: resetKeys,
                selectedGroupByCols: resetKeys,
                values: this.initializeAggregatedValues(this.props.sheetJSONArray[newSelectedSheetIndex].columns)
            });
        };
        // function used to add/remove column headers to the row and column keys 
        this.editColumnKeySelection = (keyType, columnHeader, operation) => {
            const selectedColumnIndex = this.props.sheetJSONArray[this.state.selectedSheetIndex].columns.indexOf(columnHeader);
            // sanity check: make sure that this column is not already selected. this should not be possible!
            if (operation === KeySelectionOperation.ADD) {
                if (this.state.selectedGroupByRows[selectedColumnIndex] || this.state.selectedGroupByCols[selectedColumnIndex]) {
                    console.log("column selected as key twice in pivot table. should not be possible");
                    return;
                }
            }
            // handle the row case
            if (keyType === KeyType.Row) {
                this.setState(prevState => {
                    const newSelectedGroupByRows = [...prevState.selectedGroupByRows];
                    // if the operation was add, then set index to true. otherwise set to false
                    newSelectedGroupByRows[selectedColumnIndex] = operation === KeySelectionOperation.ADD;
                    return {
                        selectedGroupByRows: newSelectedGroupByRows
                    };
                });
            }
            // handle the column case
            if (keyType === KeyType.Col) {
                this.setState(prevState => {
                    const newSelectedGroupByCols = [...prevState.selectedGroupByCols];
                    // if the operation was add, then set index to true. otherwise set to false
                    newSelectedGroupByCols[selectedColumnIndex] = operation === KeySelectionOperation.ADD;
                    return {
                        selectedGroupByCols: newSelectedGroupByCols
                    };
                });
            }
        };
        this.completeGroup = () => {
            var _a;
            /*
                Completes the group operation by sending information for the group
                to the backend.
    
                Errors if no group-by keys are selected.
            */
            const groupByRows = this.props.sheetJSONArray[this.props.selectedSheetIndex].columns.filter((columnHeader, index) => {
                return this.state.selectedGroupByRows[index];
            });
            const groupByCols = this.props.sheetJSONArray[this.props.selectedSheetIndex].columns.filter((columnHeader, index) => {
                return this.state.selectedGroupByCols[index];
            });
            // If there are no keys grouped by, we insert an error, and do not let the user submit
            if (groupByRows.length == 0 && groupByCols.length == 0) {
                this.setState({
                    keyError: 'Please select a at least one row or column before submitting'
                });
                return;
            }
            // Build the values mapping
            const values = {};
            /*
            this.state.values.forEach((value) => {
                values[value[0]] = value[1];
            })
            */
            // Log
            (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
                userId: window.user_id,
                event: 'button_group_log_event',
                properties: {
                    sheet_index: this.props.selectedSheetIndex,
                    group_by_rows: groupByRows,
                    group_by_cols: groupByCols,
                    values: values
                }
            });
            this.props.send({
                'event': 'edit_event',
                'type': 'group_edit',
                sheet_index: this.props.selectedSheetIndex,
                group_rows: groupByRows,
                group_columns: groupByCols,
                values: values
            });
            this.props.setTaskpaneOpenOrClosed(false);
        };
        const columns = this.props.sheetJSONArray[props.selectedSheetIndex].columns;
        // We default to selecting _no_ columns to group by
        const selectedGroupByRows = columns.map(() => false);
        const selectedGroupByCols = columns.map(() => false);
        // we default to selecting no values to aggregate
        const aggregatedValues = this.initializeAggregatedValues(columns);
        this.state = {
            // TODO: Update default sheet index to the page they are currently on
            selectedSheetIndex: props.selectedSheetIndex,
            selectedGroupByRows: selectedGroupByRows,
            selectedGroupByCols: selectedGroupByCols,
            values: aggregatedValues,
            keyError: ''
        };
        this.completeGroup = this.completeGroup.bind(this);
        this.getSelectableKeyColumnHeaders = this.getSelectableKeyColumnHeaders.bind(this);
        this.getGroupByKeys = this.getGroupByKeys.bind(this);
        this.editValueAggregationSelection = this.editValueAggregationSelection.bind(this);
        this.initializeAggregatedValues = this.initializeAggregatedValues.bind(this);
    }
    getSelectableKeyColumnHeaders() {
        /*
            Returns the remaining possible column header keys.
            A column header can only be used in either the row or column key
        */
        // find all column headers that are not selected
        const allUnselectedColumnHeaders = [];
        for (let i = 0; i < this.state.selectedGroupByCols.length; i++) {
            if (!this.state.selectedGroupByRows[i] && !this.state.selectedGroupByCols[i]) {
                allUnselectedColumnHeaders.push(this.props.sheetJSONArray[this.state.selectedSheetIndex].columns[i]);
            }
        }
        return allUnselectedColumnHeaders;
    }
    getGroupByKeys(keyType) {
        /*
            Returns the selected column header for the keyType
        */
        const selectedColumnHeaders = [];
        for (let i = 0; i < this.state.selectedGroupByCols.length; i++) {
            if (keyType === KeyType.Row) {
                if (this.state.selectedGroupByRows[i]) {
                    // get column header name for selected column headers in row key
                    selectedColumnHeaders.push(this.props.sheetJSONArray[this.state.selectedSheetIndex].columns[i]);
                }
            }
            if (keyType === KeyType.Col) {
                if (this.state.selectedGroupByCols[i]) {
                    // get column header name for selected column headers in column key
                    selectedColumnHeaders.push(this.props.sheetJSONArray[this.state.selectedSheetIndex].columns[i]);
                }
            }
        }
        return selectedColumnHeaders;
    }
    /*
    addValue(): void {
        
            //Adds a new value (columnHeader, aggregationFunction) mapping to this.state.values.

            //Has some heuristics to guess which columnHeader you want!
        
        this.setState(prevState => {
            const newValues = [...prevState.values];
            // We get the first column header in the array that
            const columns = this.props.sheetJSONArray[this.props.selectedSheetIndex].columns;
            // If there is no columns in this, then we have nothing to group!
            if (columns.length === 0) {
                newValues.push(['none', AGGREGATION_OPTIONS[0]])
                return {
                    values: newValues
                }
            }
            // Find the first column that has not been selected as a key, and
            // also has not been grouped too, as a heuristic for what the user
            // wants to aggregate
            const unaggregatedColumns = columns.filter((columnHeader, index) => {
                return !prevState.selectedGroupByRows[index] && prevState.values.findIndex((value) => {
                    return value[0] == columnHeader;
                }) < 0;
            })
            // Take the first unaggregated column if it exists, otherwise just take the
            // first column
            const newColumn = unaggregatedColumns.length > 0 ? unaggregatedColumns[0] : columns[0];
            newValues.push([newColumn, AGGREGATION_OPTIONS[0]])
            return {
                values: newValues
            }
        })
    }
    */
    render() {
        const selectableColumnHeaders = this.getSelectableKeyColumnHeaders();
        const groupByRows = this.getGroupByKeys(KeyType.Row);
        const groupByCols = this.getGroupByKeys(KeyType.Col);
        //const values = this.getValues()
        return (react_1.default.createElement(DefaultTaskpane_1.default, { header: 'Create a Pivot Table', isTaskpaneOpen: this.props.isTaskpaneOpen, setTaskpaneOpenOrClosed: this.props.setTaskpaneOpenOrClosed, taskpaneBody: react_1.default.createElement(react_1.Fragment, null,
                react_1.default.createElement("div", { className: 'default-taskpane-body-section-div pivot-taskpane-section-header-div' },
                    react_1.default.createElement("p", { className: 'default-taskpane-body-section-title-text' }, "Data Source"),
                    react_1.default.createElement(LargeSelect_1.default, { startingValue: this.props.dfNames[this.state.selectedSheetIndex], key: this.state.selectedSheetIndex, optionsArray: this.props.dfNames, setValue: this.setSelectedSheet })),
                react_1.default.createElement("div", { className: 'default-taskpane-body-section-div' },
                    react_1.default.createElement(PivotTableKeySelection_1.default, { sectionTitle: 'Rows', selectableColumnHeaders: selectableColumnHeaders, columnHeadersInSection: groupByRows, editColumnKeySelection: this.editColumnKeySelection, keyType: KeyType.Row })),
                react_1.default.createElement("div", { className: 'default-taskpane-body-section-div' },
                    react_1.default.createElement(PivotTableKeySelection_1.default, { sectionTitle: 'Columns', selectableColumnHeaders: selectableColumnHeaders, columnHeadersInSection: groupByCols, editColumnKeySelection: this.editColumnKeySelection, keyType: KeyType.Col })),
                react_1.default.createElement("div", { className: 'default-taskpane-body-section-div' },
                    react_1.default.createElement(PivotTableValueSelection_1.default, { aggregatedValuesInSection: this.state.values, editValueAggregationSelection: this.editValueAggregationSelection }))) }));
    }
}
exports.default = PivotTaskpane;
//# sourceMappingURL=PivotTaskpane.js.map