"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const DefaultTaskpane_1 = __importDefault(require("../../DefaultTaskpane"));
const LargeSelect_1 = __importDefault(require("../../elements/LargeSelect"));
const PivotTableKeySelection_1 = __importDefault(require("./PivotTableKeySelection"));
// import 
require("../../../../css/pivot-taskpane.css");
const PivotTableValueSelection_1 = __importDefault(require("./PivotTableValueSelection"));
// The aggregation functions we're supporting, out of the box
var AggregationType;
(function (AggregationType) {
    AggregationType["SUM"] = "sum";
    AggregationType["MEAN"] = "mean";
    AggregationType["MEDIAN"] = "median";
    AggregationType["MIN"] = "min";
    AggregationType["MAX"] = "max";
    AggregationType["COUNT"] = "count";
    AggregationType["STD"] = "std";
    AggregationType["NONE"] = "none";
})(AggregationType = exports.AggregationType || (exports.AggregationType = {}));
var KeyType;
(function (KeyType) {
    KeyType["Row"] = "Row";
    KeyType["Col"] = "Col";
})(KeyType = exports.KeyType || (exports.KeyType = {}));
var KeySelectionOperation;
(function (KeySelectionOperation) {
    KeySelectionOperation["ADD"] = "Add";
    KeySelectionOperation["REMOVE"] = "Remove";
})(KeySelectionOperation = exports.KeySelectionOperation || (exports.KeySelectionOperation = {}));
class PivotTaskpane extends react_1.default.Component {
    constructor(props) {
        super(props);
        /* create aggregated values mapping for columns with no aggregation */
        this.initializeAggregatedValues = (columnHeaders) => {
            const aggregatedValues = {};
            columnHeaders.forEach(columnHeader => {
                aggregatedValues[columnHeader] = AggregationType.NONE;
            });
            return aggregatedValues;
        };
        this.editValueAggregationSelection = (columnHeader, aggregationType) => {
            this.setState(prevState => {
                const newValues = prevState.values;
                newValues[columnHeader] = aggregationType;
                return {
                    values: newValues
                };
            }, () => {
                this.updatePivotTable(true);
            });
        };
        /*
            a callback used by the select data source Select Element so that it can
            set the state of the Pivot Table Taskpane
        */
        this.setSelectedSheet = (newSheetName) => {
            const newSelectedSheetIndex = this.state.dfNames.indexOf(newSheetName);
            /*
                reset the keys so
                    1) the column headers currently selected are dropped
                    2) the values currently aggregated are dropped
                    3) display the column headers from the new sheet
            */
            const resetKeys = this.props.sheetJSONArray[newSelectedSheetIndex].columns.map(() => { return false; });
            this.setState({
                selectedSheetIndex: newSelectedSheetIndex,
                selectedGroupByRows: resetKeys,
                selectedGroupByCols: resetKeys,
                values: this.initializeAggregatedValues(this.props.sheetJSONArray[newSelectedSheetIndex].columns)
            });
        };
        // function used to add/remove column headers to the row and column keys 
        this.editColumnKeySelection = (keyType, columnHeader, operation) => {
            const selectedColumnIndex = this.props.sheetJSONArray[this.state.selectedSheetIndex].columns.indexOf(columnHeader);
            // sanity check: make sure that this column is not already selected. this should not be possible!
            if (operation === KeySelectionOperation.ADD) {
                if (this.state.selectedGroupByRows[selectedColumnIndex] || this.state.selectedGroupByCols[selectedColumnIndex]) {
                    console.log("column selected as key twice in pivot table. should not be possible");
                    return;
                }
            }
            // handle the row case
            if (keyType === KeyType.Row) {
                this.setState(prevState => {
                    const newSelectedGroupByRows = [...prevState.selectedGroupByRows];
                    // if the operation was add, then set index to true. otherwise set to false
                    newSelectedGroupByRows[selectedColumnIndex] = operation === KeySelectionOperation.ADD;
                    return {
                        selectedGroupByRows: newSelectedGroupByRows
                    };
                }, () => {
                    this.updatePivotTable(true);
                });
            }
            // handle the column case
            if (keyType === KeyType.Col) {
                this.setState(prevState => {
                    const newSelectedGroupByCols = [...prevState.selectedGroupByCols];
                    // if the operation was add, then set index to true. otherwise set to false
                    newSelectedGroupByCols[selectedColumnIndex] = operation === KeySelectionOperation.ADD;
                    return {
                        selectedGroupByCols: newSelectedGroupByCols
                    };
                }, () => {
                    this.updatePivotTable(true);
                });
            }
        };
        this.updatePivotTable = (overwrite) => {
            /*
                Completes the group operation by sending information for the group
                to the backend.
    
                Errors if no group-by keys are selected.
            */
            var _a;
            const groupByRows = this.props.sheetJSONArray[this.state.selectedSheetIndex].columns.filter((columnHeader, index) => {
                return this.state.selectedGroupByRows[index];
            });
            const groupByCols = this.props.sheetJSONArray[this.state.selectedSheetIndex].columns.filter((columnHeader, index) => {
                return this.state.selectedGroupByCols[index];
            });
            // Build the values mapping by filtering out any columns with AggregationType.NONE
            const aggregatedValues = {};
            for (const key of Object.keys(this.state.values)) {
                if (this.state.values[key] !== AggregationType.NONE) {
                    aggregatedValues[key] = this.state.values[key];
                }
            }
            // Log
            (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
                userId: window.user_id,
                event: 'button_pivot_log_event',
                properties: {
                    sheet_index: this.state.selectedSheetIndex,
                    pivot_rows: groupByRows,
                    pivot_cols: groupByCols,
                    values: this.state.values.filter,
                    overwrite: overwrite
                }
            });
            this.props.send({
                'event': 'edit_event',
                'type': 'pivot_edit',
                sheet_index: this.state.selectedSheetIndex,
                pivot_rows: groupByRows,
                pivot_columns: groupByCols,
                values: aggregatedValues,
                overwrite: overwrite
            });
        };
        const columns = this.props.sheetJSONArray[props.selectedSheetIndex].columns;
        // We default to selecting _no_ columns to group by
        const selectedGroupByRows = columns.map(() => false);
        const selectedGroupByCols = columns.map(() => false);
        // we default to selecting no values to aggregate
        const aggregatedValues = this.initializeAggregatedValues(columns);
        this.state = {
            dfNames: this.props.dfNames,
            selectedSheetIndex: props.selectedSheetIndex,
            selectedGroupByRows: selectedGroupByRows,
            selectedGroupByCols: selectedGroupByCols,
            values: aggregatedValues,
            keyError: '',
        };
        // create a new blank sheet for the pivot table
        this.updatePivotTable(false);
        this.getSelectableKeyColumnHeaders = this.getSelectableKeyColumnHeaders.bind(this);
        this.getGroupByKeys = this.getGroupByKeys.bind(this);
        this.editValueAggregationSelection = this.editValueAggregationSelection.bind(this);
        this.initializeAggregatedValues = this.initializeAggregatedValues.bind(this);
        this.updatePivotTable = this.updatePivotTable.bind(this);
    }
    getSelectableKeyColumnHeaders() {
        /*
            Returns the remaining possible column header keys.
            A column header can only be used in either the row or column key
        */
        // find all column headers that are not selected
        const allUnselectedColumnHeaders = [];
        for (let i = 0; i < this.state.selectedGroupByCols.length; i++) {
            if (!this.state.selectedGroupByRows[i] && !this.state.selectedGroupByCols[i]) {
                allUnselectedColumnHeaders.push(this.props.sheetJSONArray[this.state.selectedSheetIndex].columns[i]);
            }
        }
        return allUnselectedColumnHeaders;
    }
    getGroupByKeys(keyType) {
        /*
            Returns the selected column header for the keyType
        */
        const selectedColumnHeaders = [];
        for (let i = 0; i < this.state.selectedGroupByCols.length; i++) {
            if (keyType === KeyType.Row) {
                if (this.state.selectedGroupByRows[i]) {
                    // get column header name for selected column headers in row key
                    selectedColumnHeaders.push(this.props.sheetJSONArray[this.state.selectedSheetIndex].columns[i]);
                }
            }
            if (keyType === KeyType.Col) {
                if (this.state.selectedGroupByCols[i]) {
                    // get column header name for selected column headers in column key
                    selectedColumnHeaders.push(this.props.sheetJSONArray[this.state.selectedSheetIndex].columns[i]);
                }
            }
        }
        return selectedColumnHeaders;
    }
    render() {
        const selectableColumnHeaders = this.getSelectableKeyColumnHeaders();
        const groupByRows = this.getGroupByKeys(KeyType.Row);
        const groupByCols = this.getGroupByKeys(KeyType.Col);
        return (react_1.default.createElement(DefaultTaskpane_1.default, { header: 'Create a Pivot Table', isTaskpaneOpen: this.props.isTaskpaneOpen, setTaskpaneOpenOrClosed: this.props.setTaskpaneOpenOrClosed, taskpaneBody: react_1.default.createElement(react_1.Fragment, null,
                react_1.default.createElement("div", { className: 'default-taskpane-body-section-div pivot-taskpane-section-header-div' },
                    react_1.default.createElement("p", { className: 'default-taskpane-body-section-title-text' }, "Data Source"),
                    react_1.default.createElement(LargeSelect_1.default, { startingValue: this.state.dfNames[this.state.selectedSheetIndex], key: this.state.selectedSheetIndex, optionsArray: this.state.dfNames, setValue: this.setSelectedSheet })),
                react_1.default.createElement("div", { className: 'default-taskpane-body-section-div' },
                    react_1.default.createElement(PivotTableKeySelection_1.default, { sectionTitle: 'Rows', selectableColumnHeaders: selectableColumnHeaders, columnHeadersInSection: groupByRows, editColumnKeySelection: this.editColumnKeySelection, keyType: KeyType.Row })),
                react_1.default.createElement("div", { className: 'default-taskpane-body-section-div' },
                    react_1.default.createElement(PivotTableKeySelection_1.default, { sectionTitle: 'Columns', selectableColumnHeaders: selectableColumnHeaders, columnHeadersInSection: groupByCols, editColumnKeySelection: this.editColumnKeySelection, keyType: KeyType.Col })),
                react_1.default.createElement("div", { className: 'default-taskpane-body-section-div' },
                    react_1.default.createElement(PivotTableValueSelection_1.default, { aggregatedValuesInSection: this.state.values, editValueAggregationSelection: this.editValueAggregationSelection }))) }));
    }
}
exports.default = PivotTaskpane;
//# sourceMappingURL=PivotTaskpane.js.map